package com.hissenit.crypto.examples.complex;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;

import com.hissenit.crypto.common.Secret;
import com.hissenit.crypto.compactencryption.CompactBinarystreamEncryption;
import com.hissenit.crypto.compactencryption.CompactPasswordKey;
import com.hissenit.crypto.compactencryption.CompactPasswordKey.GenerationParametersCPK;

public class PBEFileEncryption_CustomSettings {
	
	public static void main(String[] args) throws Exception{
		String password = "Demo Password!";
		
		//Creating a PBKDF2 based key derivation
		
		//Every single parameter can be changed or let to default
		GenerationParametersCPK parameters = CompactPasswordKey.createParameters();
		parameters.hmacAlgorithm = "HmacWhirlpool";
		parameters.iterationCount = 10000;
		parameters.keyLengthInBytes = 24;
		parameters.saltLengthInBytes = 20;
		parameters.checksumEncryptionAlgorithm = "Twofish";
		CompactPasswordKey key = CompactPasswordKey.generateKey(new Secret(password), parameters);
		
		//Creating a temp file for demonstration with some content, initializing the streams
		File inputFile = createDemoFile();
		File outputFile = File.createTempFile("crococryptlibdemo-ENCRYPTEDFILE-", ".data");
		FileInputStream fis = new FileInputStream(inputFile);
		FileOutputStream fos = new FileOutputStream(outputFile);

		//Writing the PBE/PBKDF2 parameters to the file
		key.writeToStream(fos);
		
		//ENCRYPTION
		
		//Using CBC mode (standard) with Twofish instead of AES. Keylength will be 192bit (see above)
		CompactBinarystreamEncryption cbe = new CompactBinarystreamEncryption(key, "Twofish");
		cbe.encrypt(fis, fos);
		fis.close();
		fos.close();
		
		System.out.println("Written from " + inputFile + " to " + outputFile);
		
		//DECRYPTION
		File tmp = outputFile;
		outputFile = File.createTempFile("crococryptlibdemo-DECRYPTEDFILE-", ".txt");
		inputFile = tmp;
		fis = new FileInputStream(inputFile);
		fos = new FileOutputStream(outputFile);
		
		//Reading the PBE/PBKDF2 from the stream
		key = CompactPasswordKey.newInstance(new Secret(password));
		key.readFromStream(fis);
		
		//Actual decryption
		cbe = new CompactBinarystreamEncryption(key);
		cbe.decrypt(fis, fos);
		fis.close();
		fos.close();
		
		System.out.println("Written from " + inputFile + " to " + outputFile);
	}
	
	private static final File createDemoFile() throws Exception{
		File tmp = File.createTempFile("crococryptlibdemo-ORIGINALFILE-", ".txt");
		FileOutputStream fos = new FileOutputStream(tmp);
		fos.write("Small demo file".getBytes());
		fos.flush();
		fos.close();
		return tmp;
	}
	
}
