package com.hissenit.crypto.examples.complex;

import com.hissenit.crypto.common.Secret;
import com.hissenit.crypto.compacthashing.CompactPasswordHash;
import com.hissenit.crypto.compacthashing.CompactPasswordHash.GenerationParametersCPH;


public class PasswordHash_WithCustomSettings {
	public static void main(String[] args) {
		//Provide a password via commandline or use the demo password instead
		if(args == null || args.length == 0 || args[0] == null)
			args = new String[]{"Demo Password!"};
		
		//Creating a PBKDF2 based secure password hash with custom settings
		GenerationParametersCPH parameters = CompactPasswordHash.createParameters();
		parameters.hmacAlgorithm = "HmacWhirlpool";
		parameters.hashLengthInBytes = 16;
		parameters.iterationCount = 5000;
		
		CompactPasswordHash compactHash = CompactPasswordHash.generateHash(new Secret(args[0]), parameters);
		
		//Store the compact hash format in various output formats. This includes the complete hash including
		//all necessary parameters (algorithm, salt, iteration count).
		System.out.println("\nHash (Hex): " + compactHash.writeToHex());
		System.out.println("\nHash (B64): " + compactHash.writeToBase64());
		
		byte[] raw = compactHash.writeToBytes();
		
		//Let's assume 'raw' had been written to a database and was being reloaded.
		CompactPasswordHash theHash = CompactPasswordHash.newInstance();
		theHash.readFromBytes(raw);
		
		//Perform a validation check of the newly provided password against the stored hash
		System.out.println(theHash.verifyPassword(new Secret(args[0])));
		
		//Perform a validation check that has to fail
		System.out.println(theHash.verifyPassword(new Secret("wrongPassword")));
	}
}
