package com.hissenit.crypto.examples.complex;

import com.hissenit.crypto.common.Secret;
import com.hissenit.crypto.compactencryption.CompactEncryption;
import com.hissenit.crypto.compactencryption.CompactPasswordKey;
import com.hissenit.crypto.compactencryption.EncryptionKey;
import com.hissenit.crypto.compactencryption.WrapUtil;
import com.hissenit.crypto.data.StringData;

public class PasswordKeyWrapping_PlusPasswordChange {
	public static void main(String[] args) {
		String password = "password";
		Secret secret = new Secret(password);

		//---ENCRYPTION AND KEYWRAPPING---
		
		//Securely generate a new default 256bit key, e.g., for AES
		EncryptionKey encKey = EncryptionKey.generateNewKey();
		//Create a password-based key, e.g., for an application user
		CompactPasswordKey pwKey = CompactPasswordKey.generateKey(secret);
		WrapUtil wrapper = new WrapUtil(pwKey);
		
		byte[] pwKey_atStorage = pwKey.writeToBytes();
		byte[] wrappedEncKey_atStorage = wrapper.wrapToBytes(encKey);
		
		String message = "A text message, file, binary or stream";
		String encryptedMessage = new CompactEncryption(encKey).encryptToBase64(new StringData(message));
		
		//---DECRYPTION---
		
		//Load the key material
		pwKey = CompactPasswordKey.newInstance(secret).readFromBytes(pwKey_atStorage);
		wrapper = new WrapUtil(pwKey);
		encKey = wrapper.unwrapFromBytes(wrappedEncKey_atStorage);
		
		//Perform the decryption
		System.out.println("Decrypted message:                " + new CompactEncryption(encKey).decryptFromBase64(encryptedMessage));
		
		//---REWRAPPING A KEY FOR A PASSWORD CHANGE (old and new password required)---
		
		String newPassword = "new password";
		CompactPasswordKey newPwKey = CompactPasswordKey.generateKey(new Secret(newPassword));
		//Note: The actual encryption key remains the same, although the password was changed.
		//      Hence, all encrypted data can still be decrypted! This is why there is the actual
		//      encryption key and a key encryption key (in this case password-based).
		wrappedEncKey_atStorage = wrapper.rewrapFromBytes(wrappedEncKey_atStorage, newPwKey);
		pwKey_atStorage = newPwKey.writeToBytes();
		
		//---RETRY THE DECRYPTION WITH THE REWRAPPED KEY FOR DEMONSTRATION ONLY---
		encKey = new WrapUtil(newPwKey).unwrapFromBytes(wrappedEncKey_atStorage);
		System.out.println("Decrypted message (new password): " + new CompactEncryption(encKey).decryptFromBase64(encryptedMessage));
	}
}
