package com.hissenit.crypto.examples.simple;

import com.hissenit.crypto.compactencryption.CompactEncryption;
import com.hissenit.crypto.compactencryption.EncryptionKey;
import com.hissenit.crypto.data.StringData;


public class CompactEncryptionExamples {
	public static void main(String[] args) {
		//Securely generate a raw symmetric key (e.g., for AES-256)
		EncryptionKey key = EncryptionKey.generateNewKey();
		
		System.out.println("Default keylength:     " + key.getKeyInBytes().length);
		System.out.println("Base64 encoded format: " + key.writeToBase64());
		System.out.println("Default encryption:    " + new CompactEncryption(key).getAlgorithm());
		System.out.println();

		String message = "Plain message. Could also be binary.";
		
		//The encryption includes all necessary parameters, including a securely generated IV for the CBC block cipher mode
		String encryptedMessage = new CompactEncryption(key).encryptToBase64(new StringData(message));
		System.out.println("Encrypted message:     " + encryptedMessage);
		System.out.println("Decrypted message:     " + ((StringData)new CompactEncryption(key).decryptFromBase64(encryptedMessage)).getDataString());
		System.out.println();
		
		//Every symmetric block cipher available in the JCE can be used
		String encryptedMessage2 = new CompactEncryption(key, "Blowfish").encryptToBase64(new StringData(message));
		System.out.println("Encrypted message:     " + encryptedMessage2);
		System.out.println("Decrypted message:     " + ((StringData)new CompactEncryption(key).decryptFromBase64(encryptedMessage2)).getDataString());

	}
}
