package com.hissenit.crypto.examples.simple;

import com.hissenit.crypto.common.Secret;
import com.hissenit.crypto.compactencryption.CompactPasswordKey;


public class PasswordBasedKeyExample {
	public static void main(String[] args) {
		//Provide a password via commandline or use the demo password instead
		if(args == null || args.length == 0 || args[0] == null)
			args = new String[]{"Demo Password!"};
		
		//Creating a PBKDF2 based key derivation
		CompactPasswordKey key = CompactPasswordKey.generateKey(new Secret(args[0]));
		
		//Store the compact key derivation format in various output formats. This includes the all
		//necessary parameters (algorithm, salt, iteration count) plus an encryption checksum that
		//tell's us if the password is correct
		System.out.println("Hash (Hex): " + key.writeToHex());
		System.out.println("Hash (B64): " + key.writeToBase64());
		
		byte[] binaryObject = key.writeToBytes();
		
		//Let's assume 'binaryObject' had been written to a database and was being reloaded.
		CompactPasswordKey theKey = CompactPasswordKey.newInstance(new Secret(args[0]));
		theKey.readFromBytes(binaryObject);
		
		//Perform a validation check of the newly provided password
		System.out.println(theKey.isPasswordValid());
		
		//Perform a validation check that has to fail
		theKey = CompactPasswordKey.newInstance(new Secret("wrong password"));
		theKey.readFromBytes(binaryObject);
		System.out.println(theKey.isPasswordValid());
	}
}
