package com.hissenit.crypto.examples.simple;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;

import com.hissenit.crypto.common.Secret;
import com.hissenit.crypto.compactencryption.CompactBinarystreamEncryption;
import com.hissenit.crypto.compactencryption.CompactPasswordKey;

/*
 * Simple demonstration of strong file encryption with AES-256 as library default using a password-based encryption through PBKDF2/PKCS #5.
 */
public class SimplePBEFileEncryptionExample {
	
	public static void main(String[] args) throws Exception{
		String password = "Demo Password!";
		
		//Creating a PBKDF2 based key derivation
		//DEFAULTS:
		//+ HmacSHA512
		//+ 100000 iterations
		//+ AES-256
		//+ 256bit keylength
		//+ 512bit salt
		CompactPasswordKey key = CompactPasswordKey.generateKey(new Secret(password));
		
		//Creating a temp file for demonstration with some content, initializing the streams
		File inputFile = createDemoFile();
		File outputFile = File.createTempFile("crococryptlibdemo-ENCRYPTEDFILE-", ".data");
		FileInputStream fis = new FileInputStream(inputFile);
		FileOutputStream fos = new FileOutputStream(outputFile);

		//Writing the PBE/PBKDF2 parameters to the file
		key.writeToStream(fos);
		
		//ENCRYPTION
		//DEFAULTS:
		//+ AES-256
		//+ 256bit keylength
		//+ CBC mode
		//+ 16byte initialization vector (IV)
		
		//NOTE: This class can utilize any stream, a file stream is just an example!
		CompactBinarystreamEncryption cbe = new CompactBinarystreamEncryption(key);
		cbe.encrypt(fis, fos);
		fis.close();
		fos.close();
		
		//---ENCRYPTION DONE---
		
		System.out.println("Written from " + inputFile + " to " + outputFile);
		
		//DECRYPTION
		File tmp = outputFile;
		outputFile = File.createTempFile("crococryptlibdemo-DECRYPTEDFILE-", ".txt");
		inputFile = tmp;
		fis = new FileInputStream(inputFile);
		fos = new FileOutputStream(outputFile);
		
		//Reading the PBE/PBKDF2 from the stream
		key = CompactPasswordKey.newInstance(new Secret(password));
		key.readFromStream(fis);
		
		//Actual decryption
		cbe = new CompactBinarystreamEncryption(key);
		cbe.decrypt(fis, fos);
		fis.close();
		fos.close();
		
		System.out.println("Written from " + inputFile + " to " + outputFile);
		
		//---DECRYPTION DONE---
	}
	
	private static final File createDemoFile() throws Exception{
		File tmp = File.createTempFile("crococryptlibdemo-ORIGINALFILE-", ".txt");
		FileOutputStream fos = new FileOutputStream(tmp);
		fos.write("Small demo file".getBytes());
		fos.flush();
		fos.close();
		return tmp;
	}
	
}
