package com.hissenit.crypto.examples.simple;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;

import com.hissenit.crypto.common.Secret;
import com.hissenit.crypto.compactencryption.CompactBinarystreamEncryption;
import com.hissenit.crypto.compactencryption.CompactPasswordKey;

/*
 * Simple demonstration of strong file encryption with AES-256 as library default using a password-based encryption through PBKDF2/PKCS #5.
 */
public class SimplePBEFileEncryptionExampleShort {
	
	public static void main(String[] args) throws Exception{
		String password = "Demo Password!";
		FileInputStream fis = new FileInputStream(createDemoFile());
		File outputFile = File.createTempFile("crococryptlibdemo-ENCRYPTEDFILE-", ".data");
		FileOutputStream fos = new FileOutputStream(outputFile);
		CompactPasswordKey key = CompactPasswordKey.generateKey(new Secret(password));
		key.writeToStream(fos);
		new CompactBinarystreamEncryption(key).encrypt(fis, fos);
		fis.close();
		fos.close();
		System.out.println("Encryption finished successfully: " + outputFile.getAbsolutePath());
		
		fis = new FileInputStream(outputFile);
		fos = new FileOutputStream(File.createTempFile("crococryptlibdemo-DECRYPTEDFILE-", ".txt"));
		key = CompactPasswordKey.newInstance(new Secret(password));
		key.readFromStream(fis);
		new CompactBinarystreamEncryption(key).decrypt(fis, fos);
		fis.close();
		fos.close();
		System.out.println("Decryption finished successfully: " + outputFile.getParent());
	}
	
	private static final File createDemoFile() throws Exception{
		File tmp = File.createTempFile("crococryptlibdemo-ORIGINALFILE-", ".txt");
		FileOutputStream fos = new FileOutputStream(tmp);
		fos.write("Small demo file".getBytes());
		fos.flush();
		fos.close();
		return tmp;
	}
	
}
