package com.hissenit.crypto.examples.simple;

import com.hissenit.crypto.common.Secret;
import com.hissenit.crypto.compacthashing.CompactPasswordHash;


public class SimplePasswordHashExample {
	public static void main(String[] args) {
		//Provide a password via commandline or use the demo password instead
		if(args == null || args.length == 0 || args[0] == null)
			args = new String[]{"Demo Password!"};
		
		//Creating a PBKDF2 based secure password hash with default settings
		CompactPasswordHash cph = CompactPasswordHash.generateHash(new Secret(args[0]));
		
		//Store the compact hash format in various output formats. This includes the complete hash including
		//all necessary parameters (algorithm, salt, iteration count).
		System.out.println("Hash (Hex): " + cph.writeToHex());
		System.out.println("Hash (B64): " + cph.writeToBase64());
		
		byte[] raw = cph.writeToBytes();
		
		//Let's assume 'raw' had been written to a database and was being reloaded.
		CompactPasswordHash cph_loaded = CompactPasswordHash.newInstance();
		cph_loaded.readFromBytes(raw);
		
		//Perform a validation check of the newly provided password against the stored hash
		System.out.println(cph_loaded.verifyPassword(new Secret(args[0])));
		
		//Perform a validation check that has to fail
		System.out.println(cph_loaded.verifyPassword(new Secret("wrongPassword")));
	}
}
